package GC;

import org.antlr.runtime.*;
import guicop.*;

public class GC {
  public InstrumentationParser outParser;
  public SpecParser specParser;
  public String specFile;
  public String outFile;
  public GCUDefObj spec;


  //////////// Solve: 
  //Traverse the spec declaration and find the 
  //components that match each node in the spec diagram
  //       ----------------------
  public void solve () throws Exception {
    // Note here that the leaves are solved
    // since:
    //   1- the output file is parsed and in parser.<primitive_type_name> we have all the objects that match variables of primitive_type
    //   2- constants (numbers and strings) are themselves solution objects
    spec.olog("Solving ... {");
    GCNode.sol.currentObj = spec;
    spec.match(1);
    spec.olog("}. Done solving.");
    if (spec.getMatches().isEmpty()){
      spec.olog("Testing failed: specification not met!");
    } else {
      spec.olog("Testing passed: specification satisfied.");
      spec.olog("Matching objects: {");
      spec.olog(1, spec.getMatches().toString() );
      spec.olog("}");
    }
  }

  //////////// Parse the instrumentation output file ----------------
  public void parseOut () throws Exception {
    // Create an TLexer that feeds from that stream
    InstrumentationLexer lexer = new InstrumentationLexer(new ANTLRFileStream(outFile));

    // Create a stream of tokens fed by the lexer
    CommonTokenStream tokens = new CommonTokenStream(lexer);

    // Create a parser that feeds off the token stream
    outParser = new InstrumentationParser(tokens);

    // lazy: change of name
    InstrumentationParser parser = outParser;
    parser.listofproperties();
    GCSolution sol = parser.sol;
    GCNode.sol = sol;
    //System.out.println("out = " + parser.specobjects().spec);
    spec.olog("all textrects = " + sol.textrects );
    spec.olog("all rects = " + sol.rects );
    spec.olog("all lines = " + sol.lines);
    spec.olog("all points = " + sol.points );
    spec.olog("all ellipses = " + sol.ellipses);
    spec.olog("all texts = " + sol.texts );
    spec.olog("all arcs = " + sol.arcs);
    spec.olog("all triangles= " + sol.triangles);
  }

  //////////// Parse the spec file ----------------

  public void parseSpec () throws Exception {
    // Create an TLexer that feeds from that stream
    SpecLexer lexer = new SpecLexer(new ANTLRFileStream(specFile));

    // Create a stream of tokens fed by the lexer
    CommonTokenStream tokens = new CommonTokenStream(lexer);

    // Create a parser that feeds off the token stream
    //SpecParser specParser = new SpecParser(tokens);
    specParser = new SpecParser(tokens);

    spec = specParser.specobjects().spec;
    //System.out.println("out = " + parser.specobjects().spec);
    spec.olog("all objects = " + specParser.specs);
  }



  //////////// static functions ----------------

  public static String defaultSpecFName() {
    return "./spec.gc";
  }
  public static String defaultOutFName() {
    return "./spec.gc.inst.out";
  }

  //////////// Main function ----------------

  public static void main(String[] args) throws Exception {
    GC gcop = new GC();

    if (args.length == 0) {
      gcop.specFile = defaultSpecFName();
      gcop.outFile = defaultOutFName();
    } else if (args.length == 1) {
      gcop.specFile = args[0];
      gcop.outFile = gcop.specFile + ".inst.out";
    } else if (args.length == 2) {
      gcop.specFile = args[0];
      gcop.outFile = args[1]; 
    } else if (args.length == 3) {
      gcop.specFile = args[0];
      gcop.outFile = args[1]; 
      GCNode.maxDepth = Integer.parseInt(args[2]);
      System.err.println("warning: logging up to recursion level " + args[2] + " message");

    } else {
      System.err.println("usage: program spec_file instrumentation_file max_logging_depth\n");
      return;
    }
    try{
      String [] tokens = gcop.specFile.split("/");
      String shortFileName = tokens[tokens.length -1];

      GCNode.sol = new GCSolution(shortFileName);

      gcop.parseSpec();
      gcop.parseOut();
    } catch(Exception e) {
      System.err.println(e.getMessage());
      return;
    } catch(Error err) {
      System.err.println(err.getMessage());
      return;
    }
    GCNode.sol.specs =  gcop.specParser.specs; 
    gcop.solve();
  }
}
