grammar Instrumentation;

options{
language=Java; 
backtrack=true;
}

@header{
package GC;

import java.*;
import java.util.Hashtable;
import guicop.*;
}

@lexer::header{
package GC;
}

@parser::members {
  guicop.GCSolution sol = GCNode.sol;//new guicop.GCSolution();
	GCUDefObj currentSpec;
  ArrayList<GCUDefObj> specs = new ArrayList<GCUDefObj>();
  @Override
    public void reportError(RecognitionException e) {
      super.reportError(e);
      throw new RuntimeException("instrumentation output syntax error!"); 
    }
}

@lexer::members {
  @Override
    public void reportError(RecognitionException e) {
      super.reportError(e);
      throw new RuntimeException("instrumentation output lexical error!"); 
    }
}


////////////////////////////////////////////////
// Tokens Definitions
////////////////////////////////////////////////

// Definition for integer
INT :   ('-' ('0'..'9')+) | 
         ('0'..'9')+ 
  ;

// consume whitespace
WS  :   (' ' |  '\t'|'\n'|'\r')+ {skip();} ;

DEF : QUOTE ( CHAR )* QUOTE
  ;

QUOTE : ( '\'' )
  ;

CHAR : ( ~('\r' | '\n' | '\'') | '""' );

////////////////////////////////////////////////
// Rules Definitions
////////////////////////////////////////////////
number returns [int num] : 
  INT 
  {$num= Integer.parseInt($INT.text);} ;

properties:
     ('rectangle'^ '('! rx=number ','! ry=number ','! rw=number ','! rh=number ')'! ';'!
      {
      sol.rects.add(new GCRectangleComp($rx.num, $ry.num, $rw.num, $rh.num));
      }
      )

  |  ('line'^ '('! lx1=number ','! ly1=number ','! lx2=number ','! ly2=number ')'! ';'!
      {
      sol.lines.add(new GCLineComp($lx1.num, $ly1.num, $lx2.num, $ly2.num));
      }
      )

  |  ('ellipse'^ '('! ex=number ','! ey=number ','! er1=number ','! er2=number ')'! ';'!
      {
      sol.ellipses.add(new GCEllipseComp($ex.num, $ey.num, $er1.num,$er2.num));
      }
      )

  |  ('polygon'^ '('! px1=number ','! py1=number 
      { 
      sol.polygons.add(new GCPolygonComp($px1.num,$py1.num));
      }
      (','! pxx=number ','! pyy=number
       { 
       GCPolygonComp pol = (GCPolygonComp) sol.polygons.get(sol.polygons.size()-1);
       pol.addSide($pxx.num,$pyy.num);
       }
       )* ')'! ';'!)

  |  ('triangle'^ '('! trx1=number ','! try1=number ','! trx2=number ','! try2=number ','! trx3=number ','! try3=number ')'! ';'!
      {
      sol.triangles.add(new GCTriangleComp($trx1.num,$try1.num,$trx2.num,$try2.num,$trx3.num,$try3.num));
      }
      )


  |  ('arc'^ '('! ax1=number ','! ay1=number ','! ax2=number ','! ay2=number ','! ar1=number ','! ar2=number ')'! ';'! 
      {
      sol.arcs.add(new GCArcComp($ax1.num, $ay1.num, $ax2.num, $ay2.num, $ar1.num, $ar2.num));
      }
      )


  |  ('text'^ '('! tx=number ','! ty=number ','!  ttxt=DEF ')'! ';'!
      {
      sol.texts.add(new GCTextComp($tx.num, $ty.num, $ttxt.text));
      }
      )

  |  ('textrect'^ '('! trx=number ','! tryy=number ','! trw=number ','! trh=number ','!  trtxt=DEF ')'! ';'! 
      {
      sol.textrects.add(new GCTextRectangleComp($trx.num, $tryy.num, $trw.num, $trh.num, $trtxt.text));}
      )

  |  ('point'^ '('! xpoint=number ','! ypoint=number ')'! ';'! 
      {
      sol.points.add(new GCPointComp($xpoint.num, $ypoint.num));
      }
      )
  ;  

listofproperties :
  (properties)*
  ;
