grammar Spec;

options { 
  language=Java; 
  backtrack=true;
//  k=12;
} 

@header {
package GC;
  import java.*;
  import java.util.Hashtable;
  import guicop.*;
}

@lexer::header{
package GC;
}

//
//@members {
//}
@parser::members {
	GCUDefObj currentSpec;
  ArrayList<GCUDefObj> specs = new ArrayList<GCUDefObj>();
  @Override
    public void reportError(RecognitionException e) {
      super.reportError(e);
      throw new RuntimeException("specification syntax error!"); 
    }
}

@lexer::members {
  @Override
    public void reportError(RecognitionException e) {
      super.reportError(e);
      throw new RuntimeException("specification lexical error!"); 
    }
}

ARITHMETIC_OP:
     '+'
  |  '-'
  |  '*'
  |  '/' 
  ;
  
POSITIONAL_OP:
    'leftto'
  |  'rightto'
  |  'above'
  |  'below'
  |  'contains'
  |  'over'
  |  'smaller'
  |  'leftaligned'
  |  'rightaligned'
  |  'topaligned'
  |  'bottomaligned'
  ;
  
STRING_OP:
    'issubstring'//returns a binary
  |  'textsmaller' // returns a binary
  |  'equals'// returns a binary
//  |  'concat'// removed from here since it returns a string
  ;
  
LOGIC_OP:
    'and'
  |  'or'
  |  'xor'
  |  'implies'
  ;
     
RELATIONAL_OP:
     '=='
  |  '<'
  |  '>'
  |  '!='
  ;

NOT_OP: 
        'not'
        ;


// User defined properties for the spec object
PROPERTY:  
     'X' | 'x'
  |  'Y' | 'y'
  |  'WIDTH' | 'width'
  |  'HEIGHT' | 'height'
  ;
  
  
// Definition for integer
INT  :  ('0'..'9')+
  ;

// Definition for variable ID, this can be used as the ID of the defined object or the other instantiated objects
 ID  :  ('a'..'z'|'A'..'Z'|'\_') ('a'..'z'|'A'..'Z'|'0'..'9'|'\_')*
  ;
  
// consume whitespace
WS  :   (' '|'\t'|'\n'|'\r')+ {skip();} ;

// consume comments
COMMENTS  :   ('//')~('\n'|'\r')* {skip();} ;

CHAR   :  (~('\r' | '\n' | '\'') | '""');
QUOTEDSTRING  :  '\'' (CHAR)* '\'';

////////////////////////////////////////////////
// Rules Definitions
////////////////////////////////////////////////


// Used for declaring constraints recursively
constraint returns [GCConstraint constraint]
  :  
  	boolean_expr				{$constraint = new GCConstraint($boolean_expr.expr); currentSpec.olog(".... " + $boolean_expr.expr);}
  ;

variablesdecl [Hashtable hash]
  :  ( type=ID^  var1=ID 					{$hash.put($var1.text, new GCVar($var1.text, $type.text)); //System.out.println("added " + $var1.text);
      }
	   		(','!  var2=ID				{$hash.put($var2.text, new GCVar($var2.text, $type.text)); //System.out.println("added " + $var2.text);
         }	
     					)*';'!)
  |
  ( 'flexible' type=ID^  var1=ID 					{$hash.put($var1.text, new GCVar($var1.text, $type.text, true)); //System.out.println("added " + $var1.text);
      }
	   		(','!  var2=ID				{$hash.put($var2.text, new GCVar($var2.text, $type.text, true)); //System.out.println("added " + $var2.text);
         }	
     					)*';'!)
        ;


var:
     ID 		{currentSpec.isValidVariable($ID.text);}
  ;

positional_expr returns [GCExpression expr]:
     var					{ $expr = currentSpec.getVariable($var.text);}
  |  '(' left=positional_expr op=POSITIONAL_OP right=positional_expr ')'	{$expr = GCExpression.makeBinaryExpression($op.text, $left.expr, $right.expr);}
  ;

boolean_expr returns [GCExpression expr]:
     'true'				{ $expr = new GCBoolean(true);}
  |  'false'			{ $expr = new GCBoolean(false);}
  |  positional_expr {$expr = $positional_expr.expr;}
  |  boolean_str_expr { $expr = $boolean_str_expr.expr;}
  |  boolean_ar_expr {$expr = $boolean_ar_expr.expr;}
  |  '(' leftb=boolean_expr opb=LOGIC_OP rightb=boolean_expr ')'					{$expr = GCExpression.makeBinaryExpression($opb.text, $leftb.expr, $rightb.expr);}
  ;

boolean_ar_expr returns [GCExpression expr]:
'(' lefta=arithmetic_expr opa=RELATIONAL_OP righta=arithmetic_expr ')'		{$expr = GCExpression.makeBinaryExpression($opa.text, $lefta.expr, $righta.expr);}


  |  '(' op=NOT_OP operand=boolean_expr ')'										{$expr = GCExpression.makeUnaryExpression($op.text, $operand.expr);}
  ;

boolean_str_expr returns [GCExpression expr]: 
  '(' leftstr=string_expr opstr=STRING_OP rightstr=string_expr ')'		{$expr = GCExpression.makeBinaryExpression($opstr.text, $leftstr.expr, $rightstr.expr);}
  ;
  
arithmetic_expr returns [GCExpression expr]  :
     INT                            
{ $expr = new GCNumber($INT.text);}
  |  var '.' arith_property_name 		
{ $expr = new GCMemberVariable(currentSpec.getVariable($var.text), $arith_property_name.text);}
  |  '('  left=arithmetic_expr   op=ARITHMETIC_OP  right=arithmetic_expr ')'	{$expr = GCExpression.makeBinaryExpression($op.text, $left.expr, $right.expr);}
  ;

string_expr returns [GCExpression expr]:
     QUOTEDSTRING		{ $expr = new GCString($QUOTEDSTRING.text);}
  |  var '.' string_property_name { $expr = new GCMemberVariable(currentSpec.getVariable($var.text), "text");}
  |  '(' left=string_expr op='concat' right=string_expr ')'	{$expr = GCExpression.makeBinaryExpression($op.text, $left.expr, $right.expr);}
  ;

arith_property_name returns [String name]:
	PROPERTY 				{$name = $PROPERTY.text;}
	| ID					{$name = $ID.text;}	
  ;

string_property_name returns [String name]:
	'text' {$name = "text";}
  | 'string' {$name = "text";}
	| ID					{$name = $ID.text;}	
  ;


property returns [GCProperty prop]:  
	arith_property_name '='! arithmetic_expr ';'! 	{ $prop = new GCProperty(); $prop.name = $arith_property_name.text; $prop.add($arithmetic_expr.expr);}
	| 'text' '='! string_expr ';'! 					{$prop = new GCProperty(); $prop.name = "text"; $prop.add($string_expr.expr); $prop.isText=true;}
	| 'String' ID '='! string_expr ';'! 			{ $prop = new GCProperty(); $prop.name = $ID.text; $prop.add($string_expr.expr);$prop.isText = true;}

  ;


//////////////////////////////////////////////////////////////////  
// This will be the main rule used to read the specification input
specobjects	returns [GCUDefObj spec]
  :  spec1=specobject  {specs.add($spec1.spec);} 
     (spec2=specobject {specs.add($spec2.spec);} )*	{$spec = $spec1.spec;}
  ;

specobject	returns [GCUDefObj spec]
  :  (ID^  							{currentSpec = $spec = new GCUDefObj($ID.text);} 	
		'='!  '{'!
    
    		variables   [$spec.variables]
      
      		(properties [$spec.properties] )?
      
      		constraints [$spec.constraints]
      
    '}'!
	
     )
  ;

variables [Hashtable hash]
  :  ( 'variables'^  '{'!
      variablesdecl[hash]*
      
     '}'!)
  ;

properties [ArrayList list]
  :  ( 'properties'^  '{'!
      
      (property				{$list.add($property.prop);})*

     '}'!)
  ;

constraints [ArrayList list]
  :  ( 'constraints'^  '{'!

        (constraint ';'!		{$list.add($constraint.constraint);})*

     '}'!)
  ;

