package guicop;
import java.util.*;
import java.awt.Rectangle;

public class GCCompositeComp extends GCComponent {
  GCComponent components[];
  GCUDefObj obj;
  int x;
  int y;
  int width;
  int height;
  String text;

  // iVal and strVal are mostly used for expressions in properties 
  int iVal;
  String strVal;
 

  HashMap<String,Integer> intProperties = new  HashMap<String,Integer> ();
  HashMap<String,String> textProperties = new  HashMap<String,String> ();

  public String componentsToString() {
    String str="";
    if (components.length > 0) {
      str = "<";
      for (GCComponent comp : components) {
        if (comp.isJoker()){
          str+="*,";
          continue;
        } 
        str += Integer.toString(comp.id);
        str += ",";
      }
      str += ">";
    }
    return str;
  }

  public String toString() {
    String str = "composite x:" + Integer.toString(x) +
      ",y:" +  Integer.toString(y) +
      ",w:" +  Integer.toString(width) +
      ",h:" +  Integer.toString(height) +
      ",iv:" +  Integer.toString(iVal) +
      ",sv:" +  strVal +
      ",t:" +  text + 
      ",#:" + components.length;
    str+= componentsToString();
    str+="\n";
    return str;
  }

  @Override
  public GCUDefObj getUDefObj() {return obj;}

  @Override
  public int getX() {
    if (obj.propertiesComputed) {
      return x;
    }
    return (int)getBoundingRect().getX();
  }

  @Override
  public int getY() {
    if (obj.propertiesComputed) {
      return y;
    }
    return (int)getBoundingRect().getY();
  }

  @Override
  public int getWidth() {
    if (obj.propertiesComputed) {
      return width;
    }
    return (int)getBoundingRect().getWidth();
  }

  @Override
  public int getHeight() {
    if (obj.propertiesComputed) {
      return height;
    }
    return (int)getBoundingRect().getHeight();
  }

  @Override
  public int getMinX() {
    int i=0;
    int minX=0;
    for (GCComponent comp : components) {
      if (comp.isJoker()){
        continue;
      }
      if (i == 0) {
        minX = comp.getMinX();
      } else {
        int x2 = comp.getMinX();
        if (x2 < minX) {
          minX = x2;
        }
      }
      i=i+1;
    }
    assert(i!=0);
    return minX;
  }

  @Override
  public int getMaxX() {
    int i=0;
    int maxX=0;
    for (GCComponent comp : components) {
      if (comp.isJoker()){
        continue;
      }
      if (i == 0) {
        maxX = comp.getMaxX();
      } else {
        int x2 = comp.getMaxX();
        if (x2 > maxX) {
          maxX = x2;
        }
      }
      i=i+1;
    }
    assert(i!=0);
    return maxX;
  }


  @Override
  public int getMinY() {
    int i=0;
    int minY=0;
    for (GCComponent comp : components) {
      if (comp.isJoker()){
        continue;
      }
      if (i == 0) {
        minY = comp.getMinY();
      } else {
        int y2 = comp.getMinY();
        if (y2 < minY) {
          minY = y2;
        }
      }
      i=i+1;
    }
    assert(i!=0);
    return minY;
  }

  @Override
  public int getMaxY() {
    int i=0;
    int maxY=0;
    for (GCComponent comp : components) {
      if (comp.isJoker()){
        continue;
      }
      if (i == 0) {
        maxY = comp.getMaxY();
      } else {
        int y2 = comp.getMaxY();
        if (y2 > maxY) {
          maxY = y2;
        }
      }
      i=i+1;
    }
    assert(i!=0);
    return maxY;
  }
    
  @Override
  public String getText () {return text;}
  
  // mostly used for expressions in properties 
  @Override
  public String getStringValue () {return strVal;}

  @Override
  public int getIntValue () {return iVal;}

  @Override
  public boolean isComposite () {
    return true;
  }

  public boolean isDataEqual(GCCompositeComp comp){
    if (x != comp.x) return false;
    if (y != comp.y) return false;
    if (width != comp.width) return false;
    if (height != comp.height) return false;

//  commented this out since iVal and strVal are not part of the object data, 
//  they are part of the property expression computations
//    if (!strVal.equals(comp.strVal)) return false;
//    if (iVal != comp.iVal) return false;
    if (components.length != comp.components.length) return false;

    if (text == null && comp.text == null) return true;
    if (!text.equals(comp.text)) return false;
    return true;
  }

  public GCComponent getIntersection(GCComponent _comp) {
    if (_comp.isJoker()){
      return this;
    } else if (!_comp.isComposite()){
      return null;
    }
    GCCompositeComp comp = _comp.composite();
    if (!isDataEqual(comp)){
      return null;
    }

    GCNode.solog(GCNode.sol.depth+1, "composite intersection start {");
    GCNode.solog(GCNode.sol.depth+1, componentsToString() + " /\\ " + comp.componentsToString());

    GCCompositeComp com_comp = clone();

    for (int i = 0; i < components.length; i++) {
      GCComponent c1 = components[i];
      GCComponent c2 = comp.components[i];
      if (c1.isJoker() ) {
        if (c2.isJoker()){
          // insert a joker at i
          // already there because of clone
        } else {
          // insert c2 at i
          if (!com_comp.doesAddingComponentIRespectExclusive(c2, i, components.length)){
            GCNode.solog(GCNode.sol.depth+1, "} composite intersection end. empty intersection due to exclusion");
            return null;
          }
          com_comp.components[i] = c2; 
        }
      } else {
        if (c2.isJoker()){
          // insert c1 at i
          // it is already there because of the clone
        } else {
          if (c1.isEqual(c2) ){
            // insert c1 at i
            // already there because of clone
          } else {
            //should never happen. 
            //throw result away and return null
            GCNode.solog(GCNode.sol.depth+1, "} composite intersection end. empty intersection");
            return null;
          }
        }
      }
    }
    GCNode.solog(GCNode.sol.depth+1, "} composite intersection end" +  
        com_comp.componentsToString() + ".");
    return com_comp;
  }

  @Override
  public boolean isEqual(GCComponent _comp) {
    if (_comp.isJoker()) {
      GCNode.solog(GCNode.sol.depth+1, "composite isEqual: _comp is joker");
      return true;
    } else if (!_comp.isComposite()){
      GCNode.solog(GCNode.sol.depth+1, "composite isEqual: _comp is not composite");
      return false;
    }

    if (id == _comp.id) {
      return true;
    }

    GCCompositeComp comp = _comp.composite();
    if (!isDataEqual(comp)){
      GCNode.solog(GCNode.sol.depth+1, "composite isEqual: data of comp and _comp are not equal ");
      return false;
    }

    GCNode.solog(GCNode.sol.depth+1, "composite isEqual start {");
    GCNode.solog(GCNode.sol.depth+1, componentsToString() + " ?= " + comp.componentsToString());
    

    for (int i = 0; i < components.length; i++) {
      GCComponent c1 = components[i];
      GCComponent c2 = comp.components[i];
      GCNode.solog(GCNode.sol.depth+1, " c1:" + c1 + " vs  c2 " + c2);
      if (!c1.isEqual(c2)) {// is Equal takes care of a joker 
                            // component well. 
        GCNode.solog(GCNode.sol.depth+1, "} composite isEqual End. found conflict");
        return false;
      }
    }
    GCNode.solog(GCNode.sol.depth+1, "} composite isEqual End. no conflict");
    return true;
  }

  public boolean equals(GCCompositeComp comp) {
    // differs from isEqual since there we 
    // need to take care of jokers
    if (!isDataEqual(comp)){
      return false;
    }

    for (int i = 0; i < components.length; i++) {
      GCComponent c1 = components[i];
      GCComponent c2 = comp.components[i];
      //HERE is the difference
      if (!c1.equals(c2)) {
        return false;
      }
    }
    return true;
  }

  public GCCompositeComp clone() {
     GCCompositeComp comp = new GCCompositeComp();
     comp.components = new GCComponent[obj.variables.size()];
     for (int i = 0; i < obj.variables.size(); i++) {
       comp.components[i] = components[i];
     }
     comp.obj = obj;
     comp.x = x;
     comp.y = y;
     comp.width = width;
     comp.height = height;
     comp.text = text;
     comp.intProperties = (HashMap<String,Integer>) intProperties.clone();
     comp.textProperties = (HashMap<String,String>) textProperties.clone();
     return comp;
  }

  // The following function checks whether adding component comp, at 
  // index i respects the flexible/exclusive constraints of the variables
  // The function assumes that all entries in components[0 .. i-1] respect
  // the constraint and therefore checks only the to be added to ith component
  // against the existing ones. 
  public boolean doesAddingComponentIRespectExclusive (GCComponent comp, int i, int n) {
    assert(i < obj.variables.size());
    assert(n <= obj.variables.size());

    if (components[i] != null && !components[i].isJoker()){
      // position already taken
      return false;
    }

    GCVar v = obj.getVarByIndex(i);
    // if the variable corresponding to the component is flexible, then 
    // return true.
    if (v.isFlexible) {
      return true;
    }

    for (int j = 0; j < n; j++) {
      if (j == i ) {
        continue;
      }
      if (components[j].isJoker()){
        continue;
      }
      if (comp.id == components[j].id) {
        // if the components are equal and variable at i is not flexible
        // then either variable at j is flexible or return false
        v = obj.getVarByIndex(j);
        if (v.isFlexible) {
          continue;
        }
        return false;
      }
    }
    // all members of components with comp at components[i] respect
    // the flexibility/exclusivity constraint
    return true;
  }

  public boolean isExclusive() {
     for (int i = 0; i < obj.variables.size(); i++) {
       GCVar v1 = obj.getVarByIndex(i);
       if (v1.isFlexible) {
         continue;
       }
       GCComponent comp1 = components[i];

       for (int j = i+1; j < obj.variables.size(); j++) {
         GCVar v2 = obj.getVarByIndex(j);
         if (v2.isFlexible) {
           continue;
         }
         GCComponent comp2 = components[j];
         if (comp1.id == comp2.id) {
           return false;
         }
       }
     }
     return true;
  }


  @Override
  public int getIntProperty(String name) {
    return intProperties.get(name);}
  @Override
  public String getStringProperty(String name) {
    return textProperties.get(name);}

  public GCCompositeComp composite() {return this;}

  GCCompositeComp () {
    this.obj = GCNode.sol.currentObj;
    assert(obj != null);// ab object in solution should exist
    components = new GCComponent[obj.variables.size()];
    for(int i = 0; i < obj.variables.size();i++){
      components[i] = GCComponent.joker;
    }
  }

  //Constructs a composite component with one component
  //  at index and the rest are jokers.
  //This is typically called for a leaf in a user defined object
  //  such as a variable declaration. 
  GCCompositeComp (GCUDefObj obj, int index, GCComponent comp) {
    this.obj = obj;
    components = new GCComponent[obj.variables.size()];
    for(int i = 0; i < obj.variables.size();i++){
      if (i == index) {

        components[i] = comp;
        x = comp.getX();
        y = comp.getY();
        width = comp.getWidth();
        height = comp.getHeight();
        text = comp.getText();
        //
        //System.out.println("X " + Integer.toString(x)  
        //    + "Y " + Integer.toString(y)  
        //    + "W " + Integer.toString(width)  
        //    + "H " + Integer.toString(height)
        //    + "T " + text);
      } else {
        components[i] = GCComponent.joker;
      }
    }
  }

  public String getConcatenatedText() {
    // returns the text in all components concatenated.
    String str = new String();
    for (GCComponent comp : components) {
//      str = str + " " + comp.getText();
      str = str + comp.getText();
    }
    return str;
  }

  // this function sets x,y,width and height to the 
  //  bounding rectangle of all components[] 
  public Rectangle getBoundingRect() {
    int i = 0;
    int x1=0, x2=0, y1=0, y2=0;
    int x1i=0, x2i=0, y1i=0, y2i=0;
    for (GCComponent comp : components) {
      if (comp.isJoker()){
        continue;
      }
      if (i == 0) {
        x1 = comp.getX();
        y1 = comp.getY();
        x2 = x1 + comp.getWidth();
        y2 = y1 + comp.getHeight();
      } else {
        x1i = comp.getX();
        y1i = comp.getY();
        x2i = x1 + comp.getWidth();
        y2i = y1 + comp.getHeight();

        if (x1i < x1) { x1 = x1i;}
        if (y1i < y1) { y1 = y1i;}

        if (x2i > x2) { x2 = x2i;} 
        if (y2i > y2) { y2 = y2i;} 
      }
      i=i+1;
    }
    assert(i!=0);
    Rectangle r = new Rectangle(x1,y1,x2-x1,y2-y1);
    return r;
  }

  //Constructs a composite component from two other
  // composite components that matched an operation
  GCCompositeComp (GCCompositeComp comp1, GCCompositeComp comp2) throws Exception {
    assert(comp1!=null);
    assert(comp2!=null);
    assert(comp1.obj!=null);
    assert(comp1.obj==comp2.obj);

    obj = comp1.obj;
    components = new GCComponent[obj.variables.size()];

    for (int i = 0; i < obj.variables.size(); i++) {
      components[i] = null;
      if (comp1.components[i] == joker) {
        if (comp2.components[i] == joker) {
          // if both are jokers, then ith is also a joker
          components[i] = joker;
        } else {
          if(!doesAddingComponentIRespectExclusive(comp2.components[i], i, i)){
            throw new Exception("Potential composite component not exclusive");
          }
          components[i] = comp2.components[i];
        }
      } else {
        if (comp2.components[i] == joker) {
          if(!doesAddingComponentIRespectExclusive(comp1.components[i], i, i)){
            throw new Exception("Potential composite component not exclusive");
          }
          components[i] = comp1.components[i];
        } else {

          // if both are not jokers
          if (comp1.components[i] == comp2.components[i]){
            // the check here should be definitely true, but anyway, no harm in making it
            if(!doesAddingComponentIRespectExclusive(comp1.components[i], i, i)){
              throw new Exception("Potential composite component not exclusive");
            }
            // both not a joker but equal, keep it
            components[i] = comp1.components[i];
          } else {
            // match condition, but conflict in position
            // throw an exception to avoid construction
            throw new Exception("Potential composite component conflicts");
          }
        }
      }
      assert(components[i] != null);
    }// end for
  }

}
