package guicop;
import java.util.*;

abstract public class GCExpression extends GCNode {

	public GCExpression()
	{
	}

  GCExpression (GCNode left ) {
    super(left);
  }

  GCExpression (GCNode left, GCNode right) {
    super(left, right);
  }
 
	public static GCExpression makeUnaryExpression(String op, GCNode operand)
	{
		if (op.equals("not"))
		{
			return new GCExpressionNot(operand);
		}
		else {
			operand.elog("Unknown operator: " + op);
			return null;
		}
	}

  public BitSet getIndicesOfExpVars(BitSet who, int depth) {
    return who;
  }

  public String computeText(GCCompositeComp composite, int depth) {
    return "";//default implementation for non-textual expressions
  }
  abstract int computeInt(GCCompositeComp composite, int depth);
  //int computeInt(GCCompositeComp composite, int depth){
  //  return 0;
  //}

	public static GCExpression makeBinaryExpression(String op, GCNode left, GCNode right)
	{
    // arithmetic: done
		if (op.equals("+")) {
			return new GCExpressionPlus(left, right);
		} else if (op.equals("*")) {
			return new GCExpressionMultiply(left, right);
		} else if (op.equals("-")) {
			return new GCExpressionMinus(left, right);
		} else if (op.equals("/")) {
			return new GCExpressionDiv(left, right);
		} else 
    // relational: done
    if (op.equals("<")) {
			return new GCExpressionLessThan(left, right);
    } else if (op.equals(">")) {
			return new GCExpressionBiggerThan(left, right);
    } else if (op.equals("==")) {
			return new GCExpressionEqual(left, right);
    } else if (op.equals("!=")) {
			return new GCExpressionDiff(left, right);
		} else 
    //positional: done
    //leftto, above, rightto, below, over, : similar to above???
    //leftaligned, rightaligned, topaligned, bottomaligned 
    //smaller, contains, 
    if (op.equals("above") ){
			return new GCExpressionAbove(left, right);
		} else if (op.equals("leftto")) {
			return new GCExpressionLeftto(left, right);
		} else if (op.equals("rightto")) {
			return new GCExpressionRightto(left, right);
		} else if (op.equals("below")) {
			return new GCExpressionBelow(left, right);
		} else if (op.equals("over")) {
			return new GCExpressionAbove(left, right);
		} else if (op.equals("leftaligned")) {
			return new GCExpressionLeftAligned(left, right);
		} else if (op.equals("rightaligned")) {
			return new GCExpressionRightAligned(left, right);
		} else if (op.equals("topaligned")) {
			return new GCExpressionTopAligned(left, right);
		} else if (op.equals("bottomaligned")) {
			return new GCExpressionBottomAligned(left, right);
		} else if (op.equals("smaller")) {
			return new GCExpressionSmaller(left, right);
		} else if (op.equals("contains")) {
			return new GCExpressionContains(left, right);
    } else
      
    //TODO: // boolean: and, or, xor, implies
    //NOT is implemented as a unary operation on its own
    if (op.equals("and")) {
      return new GCExpressionAnd(left,right);
    } else if (op.equals("or")) {
      return new GCExpressionOr(left,right);
    } else if (op.equals("xor")) {
      assert(false);
      return null;
    } else if (op.equals("implies")) {
      return new GCExpressionImplies(left,right);
    } else 

    // textual
    if (op.equals("concat")) {
			return new GCExpressionConcat(left, right);
		} else if (op.equals("issubstring")) {
			return new GCExpressionSubstring(left, right);
		} else if (op.equals("textsmaller")) {
			return new GCExpressionSmallerText(left, right);
		} else if (op.equals("equals")) {
      //TODO: check if this works fine for strings
			return new GCExpressionStrEqual(left, right);
		} else {
			left.elog("Unknow operator " + op);
			return null;
		}
	}
}
