package guicop;
import java.util.*;

public class GCExpressionAbove extends GCBinaryOperator {

	public GCExpressionAbove(GCNode left, GCNode right) {
    super(left, right);
	}

  @Override
  public String getPrettyName() {
    return "above";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {
    BitSet lWho = new BitSet(composite.obj.variables.size());
    GCExpression lExpr = (GCExpression) getLeft();
    lWho = lExpr.getIndicesOfExpVars(lWho, 0);

    BitSet rWho = new BitSet(composite.obj.variables.size());
    GCExpression rExpr = (GCExpression) getRight();
    lWho = rExpr.getIndicesOfExpVars(rWho, 0);

    int maxLY = 0;
    int minRY = 1000000;

    for (int i = 0; i < composite.obj.variables.size(); i++) {
      GCComponent comp = composite.components[i];
      if (lWho.get(i) && comp.getY() > maxLY) {
        maxLY = comp.getY();
      }
      if (rWho.get(i) && comp.getY() < minRY) {
        minRY = comp.getY();
      }
    }
    return maxLY < minRY ? 1 : 0;
  }
 
 @Override
  public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
    for (GCComponent comp1 : leftSolution) {
      for (GCComponent comp2 : rightSolution) {
        olog(GCNode.sol.depth, "comp1.y " + Integer.toString(comp1.getMaxY()) + 
            " < comp2.Y " + Integer.toString(comp2.getMinY()));
        if (comp1.isAbove(comp2)) {
          try {
            GCComponent composite = new GCCompositeComp(comp1.composite(), comp2.composite());
            matches.add(composite);
          } catch (Exception e) {
            olog(GCNode.sol.depth,"conflict in creating composite object");
          }
        }
      }
    }
    return matches;
  }

	public String toString() { return getLeft() + " Above " + getRight() + "\n";}
	
}
