package guicop;
import java.util.*;

public class GCExpressionAnd extends GCBinaryOperator {

	public GCExpressionAnd(GCNode left, GCNode right) {
    super(left, right);
	}

  @Override
  public String getPrettyName() {
    return "and";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {

    GCExpression lExpr = (GCExpression) getLeft();
    int res1 = lExpr.computeInt(composite, depth+1);
    if (res1 == 0) {
      return 0;
    }

    GCExpression rExpr = (GCExpression) getRight();
    int res2 = rExpr.computeInt(composite, depth+1);
    if (res2 == 0) {
      return 0;
    }
    return 1;
  }

	@Override
	public ArrayList<GCComponent> matchExecute(int depth) {
	    assert(children.size() == 2);
      olog(depth,"matchExecute");
      ArrayList<GCComponent> leftSolution = children.get(0).match(depth+1);
      if (leftSolution.isEmpty()) {
        return matches;
      }
      ArrayList<GCComponent> rightSolution = children.get(1).match(depth+1);
      if (rightSolution.isEmpty()) {
        return matches;
      }
      ArrayList<GCComponent> result = executeOp(leftSolution, rightSolution);
      olog(depth,"done matchExecute. found " + result.size() + " objects");
      return result;
  }
    
 @Override
 public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
   for (GCComponent comp1 : leftSolution) {
     for (GCComponent comp2 : rightSolution) {
       olog(GCNode.sol.depth, "comp1 " + comp1 + "  comp2.y " + comp2);
       if (comp1.isComposite()){
         // and is an intersection for composites
         // since a composite with a joker is really a set
          GCCompositeComp ccomp = comp1.composite();
          GCComponent comp = ccomp.getIntersection(comp2);
          if(comp != null) {
            matches.add(comp);
          }
       } else if (comp1.isEqual(comp2)) {
         matches.add(comp1);
       }
     }
   }
   return matches;
 }

	public String toString() { return getLeft() + " And " + getRight() + "\n";}
	
}
