package guicop;
import java.util.*;

public class GCExpressionBottomAligned extends GCBinaryOperator {

	public GCExpressionBottomAligned(GCNode left, GCNode right) {
    super(left, right);
	}

  @Override
  public String getPrettyName() {
    return "bottomaligned";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {
    BitSet lWho = new BitSet(composite.obj.variables.size());
    GCExpression lExpr = (GCExpression) getLeft();
    lWho = lExpr.getIndicesOfExpVars(lWho, 0);

    BitSet rWho = new BitSet(composite.obj.variables.size());
    GCExpression rExpr = (GCExpression) getRight();
    lWho = rExpr.getIndicesOfExpVars(rWho, 0);

    int maxLY = 0;
    int maxRY = 0;

    for (int i = 0; i < composite.obj.variables.size(); i++) {
      GCComponent comp = composite.components[i];
      if (lWho.get(i) && comp.getYPlusH() > maxLY) {
        maxLY = comp.getYPlusH();
      }
      if (rWho.get(i) && comp.getYPlusH() > maxRY) {
        maxRY = comp.getYPlusH();
      }
    }
    return maxLY == maxRY ? 1 : 0;
  }
 
 @Override
  public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
    for (GCComponent comp1 : leftSolution) {
      for (GCComponent comp2 : rightSolution) {
        olog(GCNode.sol.depth, "comp1.y+h " + Integer.toString(comp1.getYPlusH()) + " < comp2.y+h" + Integer.toString(comp2.getYPlusH()));
        if (comp1.getYPlusH() == comp2.getYPlusH()) {
          try {
            GCComponent composite = new GCCompositeComp(comp1.composite(), comp2.composite());
            matches.add(composite);
          } catch (Exception e) {
            olog(GCNode.sol.depth,"conflict in creating composite object");
          }
        }
      }
    }
    return matches;
  }

	public String toString() { return getLeft() + " BottomAligned " + getRight() + "\n";}
	
}
