package guicop;
import java.util.*;

public class GCExpressionContains extends GCBinaryOperator {

	public GCExpressionContains(GCNode left, GCNode right) {
    super(left, right);
	}

  @Override
  public String getPrettyName() {
    return "contains";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {
    BitSet lWho = new BitSet(composite.obj.variables.size());
    GCExpression lExpr = (GCExpression) getLeft();
    lWho = lExpr.getIndicesOfExpVars(lWho, 0);

    BitSet rWho = new BitSet(composite.obj.variables.size());
    GCExpression rExpr = (GCExpression) getRight();
    lWho = rExpr.getIndicesOfExpVars(rWho, 0);

// get minX, minY, maxX, maxY for both then return whether 
//   check if (
//   minLX <= minRX && maxLX >= maxRX && 
//   minLY <= minRY && maxRX >= maxRY

    int minLX = 1000000, minLY = 1000000;
    int minRX = 1000000, minRY = 1000000;

    int maxLX = 0, maxLY = 0;
    int maxRX = 0, maxRY = 0;

    for (int i = 0; i < composite.obj.variables.size(); i++) {
      GCComponent comp = composite.components[i];
      if (lWho.get(i)) {
        if (comp.getX() < minLX) {
          minLX = comp.getX();
        }
        if (comp.getY() < minLY) {
          minLY = comp.getY();
        }

        if (comp.getXPlusW() > maxLX) {
          maxLX = comp.getXPlusW();
        }
        if (comp.getYPlusH() > maxLY) {
          maxLY = comp.getYPlusH();
        }
      }
      if (rWho.get(i)) {
        if (comp.getX() < minRX) {
          minRX = comp.getX();
        }
        if (comp.getY() < minRY) {
          minRY = comp.getY();
        }

        if (comp.getXPlusW() > maxRX) {
          maxRX = comp.getXPlusW();
        }
        if (comp.getYPlusH() > maxRY) {
          maxRY = comp.getYPlusH();
        }
      }
    }
    boolean doesLeftContainRight = (minLX <= minRX) && (maxLX >= maxRX) && (minLY <= minRY) && (maxRX >= maxRY) ;
    return doesLeftContainRight ? 1 : 0;
  }
 
 @Override
  public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
    for (GCComponent comp1 : leftSolution) {
      for (GCComponent comp2 : rightSolution) {
        olog(GCNode.sol.depth, "comp1 " + comp1 + "  comp2.y " + comp2);
        // an object should not contain itself!
        if (comp1.isEqual(comp2)) {
          continue;
        }
        if (comp1.contains(comp2)) {
          try {
            GCComponent composite = new GCCompositeComp(comp1.composite(), comp2.composite());
            matches.add(composite);
          } catch (Exception e) {
            olog(GCNode.sol.depth,"conflict in creating composite object");
          }
        }
      }
    }
    return matches;
  }

	public String toString() { return getLeft() + " Contains " + getRight() + "\n";}
	
}
