package guicop;
import java.util.*;

public class GCExpressionImplies extends GCBinaryOperator {
  GCExpression notLeft;
  GCExpression notLeftOrRight;

	public GCExpressionImplies (GCNode left, GCNode right) {
    super(left, right);
    notLeft = new GCExpressionNot(left);
    notLeftOrRight = new GCExpressionOr(notLeft,right);
	}

  @Override
  public String getPrettyName() {
    return "implies";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {

    GCExpression lExpr = (GCExpression) getLeft();
    int res1 = lExpr.computeInt(composite, depth+1);
    if (res1 == 0) {
      return 1;
    }

    GCExpression rExpr = (GCExpression) getRight();
    int res2 = rExpr.computeInt(composite, depth+1);
    if (res2 == 1) {
      return 1;
    }
    return 0;
  }

	@Override
  public ArrayList<GCComponent> matchExecute(int depth) {
    assert(children.size() == 2);
    olog(depth,"matchExecute");

    //notLeftOrRight.matchExecute(depth+1);
	 GCExpression lExpr = (GCExpression) getLeft();

	  ArrayList<GCComponent> leftSolution = lExpr.match(depth+1);
      if (leftSolution.isEmpty()) {
        return GCNode.sol.currentObj.computeAllSolutions() ;
      }

	  GCExpression rExpr = (GCExpression) getRight();
      ArrayList<GCComponent> rightSolution = rExpr.match(depth+1);
      if (rightSolution.isEmpty()) {
        return matches;
      }
      ArrayList<GCComponent> result = executeOp(leftSolution, rightSolution);
      olog(depth,"done matchExecute. found " + result.size() + " objects");
      return result;
  }

 @Override
 public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
  for (GCComponent comp1 : leftSolution) {
     for (GCComponent comp2 : rightSolution) {
        if (comp1.isComposite()){
         // and is an intersection for composites
         // since a composite with a joker is really a set
          GCCompositeComp ccomp = comp1.composite();
          GCComponent comp = ccomp.getIntersection(comp2);
          if(comp != null) {
            matches.add(comp);
          }
       } else if (comp1.isEqual(comp2)) {
         matches.add(comp1);
       }
     }
   }
   return matches;
 }

 public String toString() { return getLeft() + " implies " + getRight() + "\n";}
	
}
