package guicop;
import java.util.*;

public class GCExpressionRightAligned extends GCBinaryOperator {

	public GCExpressionRightAligned(GCNode left, GCNode right) {
    super(left, right);
	}

  @Override
  public String getPrettyName() {
    return "rightaligned";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {
    BitSet lWho = new BitSet(composite.obj.variables.size());
    GCExpression lExpr = (GCExpression) getLeft();
    lWho = lExpr.getIndicesOfExpVars(lWho, 0);

    BitSet rWho = new BitSet(composite.obj.variables.size());
    GCExpression rExpr = (GCExpression) getRight();
    lWho = rExpr.getIndicesOfExpVars(rWho, 0);

    int maxLX = 0;
    int maxRX = 0;

    for (int i = 0; i < composite.obj.variables.size(); i++) {
      GCComponent comp = composite.components[i];
      if (lWho.get(i) && comp.getXPlusW() > maxLX) {
        maxLX = comp.getXPlusW();
      }
      if (rWho.get(i) && comp.getXPlusW() > maxRX) {
        maxRX = comp.getXPlusW();
      }
    }
    return maxLX == maxRX ? 1 : 0;
  }
 
 @Override
  public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
    for (GCComponent comp1 : leftSolution) {
      for (GCComponent comp2 : rightSolution) {
        olog(GCNode.sol.depth, "comp1.x+w " + Integer.toString(comp1.getX()) + " < comp2.x+w " + Integer.toString(comp2.getX()));
        if (comp1.getXPlusW() == comp2.getXPlusW()) {
          try {
            GCComponent composite = new GCCompositeComp(comp1.composite(), comp2.composite());
            matches.add(composite);
          } catch (Exception e) {
            olog(GCNode.sol.depth,"conflict in creating composite object");
          }
        }
      }
    }
    return matches;
  }

	public String toString() { return getLeft() + " RightAligned " + getRight() + "\n";}
	
}
