package guicop;
import java.util.*;

public class GCExpressionSmaller extends GCBinaryOperator {

	public GCExpressionSmaller(GCNode left, GCNode right) {
    super(left, right);
	}

  @Override
  public String getPrettyName() {
    return "smaller";
  }

  @Override
  public int computeInt(GCCompositeComp composite, int depth) {
    BitSet lWho = new BitSet(composite.obj.variables.size());
    GCExpression lExpr = (GCExpression) getLeft();
    lWho = lExpr.getIndicesOfExpVars(lWho, 0);

    BitSet rWho = new BitSet(composite.obj.variables.size());
    GCExpression rExpr = (GCExpression) getRight();
    lWho = rExpr.getIndicesOfExpVars(rWho, 0);

// get minX, minY, maxX, maxY for both then return whether 
//   area(1) < area(2)
//   where area is (maxX -minX) * (maxY - minY)

    int minLX = 1000000, minLY = 1000000;
    int minRX = 1000000, minRY = 1000000;

    int maxLX = 0, maxLY = 0;
    int maxRX = 0, maxRY = 0;

    for (int i = 0; i < composite.obj.variables.size(); i++) {
      GCComponent comp = composite.components[i];
      if (lWho.get(i)) {
        if (comp.getX() < minLX) {
          minLX = comp.getX();
        }
        if (comp.getY() < minLY) {
          minLY = comp.getY();
        }

        if (comp.getXPlusW() > maxLX) {
          maxLX = comp.getXPlusW();
        }
        if (comp.getYPlusH() > maxLY) {
          maxLY = comp.getYPlusH();
        }
      }
      if (rWho.get(i)) {
        if (comp.getX() < minRX) {
          minRX = comp.getX();
        }
        if (comp.getY() < minRY) {
          minRY = comp.getY();
        }

        if (comp.getXPlusW() > maxRX) {
          maxRX = comp.getXPlusW();
        }
        if (comp.getYPlusH() > maxRY) {
          maxRY = comp.getYPlusH();
        }
      }
    }
    int areaL = (maxLX - minLX)*(maxLY-minLY);
    int areaR = (maxRX - minRX)*(maxRY-minRY);
    return areaL < areaR ? 1 : 0;
  }
 
 @Override
  public ArrayList <GCComponent> executeOp(ArrayList<GCComponent > leftSolution , ArrayList<GCComponent > rightSolution) {
    for (GCComponent comp1 : leftSolution) {
      for (GCComponent comp2 : rightSolution) {
        olog(GCNode.sol.depth, "comp1.area " + Integer.toString(comp1.getArea()) + " < comp2.y " + Integer.toString(comp2.getArea()));
        if (comp1.getArea() < comp2.getArea()) {
          try {
            GCComponent composite = new GCCompositeComp(comp1.composite(), comp2.composite());
            matches.add(composite);
          } catch (Exception e) {
            olog(GCNode.sol.depth,"conflict in creating composite object");
          }
        }
      }
    }
    return matches;
  }

	public String toString() { return getLeft() + " Smaller " + getRight() + "\n";}
	
}
